.md-typeset__table {
   min-width: 100%;
}

.md-typeset table:not([class]) {
    display: table;
}

:root {
  --md-primary-fg-color:            #00004E;
  --md-primary-fg-color--light:     #0043AF;
  --md-accent-fg-color:             #FF3333;
}

h1, h2, h3 {
    color: var(--md-primary-fg-color--light);
}

.md-footer {
    background-color: #000000;
}

.md-typeset a{
    color: var(--md-primary-fg-color--light);
}

.md-nav__item .md-nav__link--active {
    color: var(--md-primary-fg-color--light);
}

@media screen and (min-width: 60em) {
    .md-search__input {
        width: 100%;
        height: 1.8rem;
        padding-left: 2.2rem;
        color: inherit;
        font-size: .8rem;
        background-color: rgba(255,255,255,.15);
        border-radius: .1rem;
    }
}
